@@ -30,6 +30,9 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <pthread.h>
+#if defined(__linux__)
+#include <sys/prctl.h>
+#endif
 #include <string.h>
 #include <errno.h>
 #include <stdbool.h>
@@ -293,7 +296,7 @@ static int onas_ddd_unwatch_hierarchy(const char *pathname, size_t len, int fd,
     if (type & ONAS_IN) {
         wd = hnode->wd;
 
-        if (!inotify_rm_watch(fd, wd)) return CL_EARG;
+        if (!inotify_rm_watch(fd, wd) && errno != ENOENT) return CL_EARG;
 
         /* Unlink the hash node from the watch descriptor lookup table */
         hnode->wd = 0;
@@ -359,6 +362,18 @@ cl_error_t onas_enable_inotif_ddd(struct onas_context **ctx)
 
 void *onas_ddd_th(void *arg)
 {
+    /* Set thread name for profiling and debuging */
+    const char thread_name[] = "clamonacc-ddd";
+
+#if defined(__linux__)
+    /* Use prctl instead to prevent using _GNU_SOURCE flag and implicit declaration */
+    prctl(PR_SET_NAME, thread_name);
+#elif defined(__APPLE__) && defined(__MACH__)
+    pthread_setname_np(thread_name);
+#else
+    logg("^ClamInotif: Setting of the thread name is currently not supported on this system\n");
+#endif
+
     struct onas_context *ctx = (struct onas_context *)arg;
     sigset_t sigset;
     const struct optstruct *pt;
@@ -381,14 +396,16 @@ void *onas_ddd_th(void *arg)
     sigdelset(&sigset, SIGUSR1);
     sigdelset(&sigset, SIGUSR2);
     /* The behavior of a process is undefined after it ignores a
-	 * SIGFPE, SIGILL, SIGSEGV, or SIGBUS signal */
+     * SIGFPE, SIGILL, SIGSEGV, or SIGBUS signal */
     sigdelset(&sigset, SIGFPE);
     sigdelset(&sigset, SIGILL);
+    sigdelset(&sigset, SIGSEGV);
     sigdelset(&sigset, SIGTERM);
     sigdelset(&sigset, SIGINT);
 #ifdef SIGBUS
     sigdelset(&sigset, SIGBUS);
 #endif
+    pthread_sigmask(SIG_SETMASK, &sigset, NULL);
 
     logg("*ClamInotif: starting inotify event loop ...\n");
 
@@ -629,46 +646,57 @@ void *onas_ddd_th(void *arg)
 
                 event = (const struct inotify_event *)p;
                 wd    = event->wd;
-                path  = wdlt[wd];
+                if (wd >= 0)
+                    path = wdlt[wd];
+                else
+                    path = NULL;
                 child = event->name;
 
                 if (path == NULL) {
-                    logg("*ClamInotif: watch descriptor not found in lookup table ... skipping\n");
+                    logg("*ClamInotif: watch descriptor (wd:%d) not found in lookup table ... skipping\n", wd);
                     continue;
                 }
 
-                len              = strlen(path);
-                size_t size      = strlen(child) + len + 2;
-                char *child_path = (char *)cli_malloc(size);
-                if (child_path == NULL) {
-                    logg("*ClamInotif: could not allocate space for child path ... aborting\n");
-                    return NULL;
-                }
-
-                if (path[len - 1] == '/') {
-                    snprintf(child_path, --size, "%s%s", path, child);
+                if (event->mask & IN_UNMOUNT) {
+                    logg("!ClamInotif: inofify event IN_UNMOUNT (mask:%d) occured, clamonacc should be restartet because a filesystem monitored by inotify was umounted.\n", event->mask);
+                } else if (event->mask & IN_Q_OVERFLOW) {
+                    logg("!ClamInotif: inotify event IN_Q_OVERFLOW (mask:%d) occured, clamonacc should be restartet because a inotify events were dropped by the kernel and the internal clamonacc inotify data structures are likely invalid.\n", event->mask);
+                } else if (event->mask & IN_IGNORED) {
+                    // Ignore for debugging purposes
                 } else {
-                    snprintf(child_path, size, "%s/%s", path, child);
+                    len              = strlen(path);
+                    size_t size      = strlen(child) + len + 2;
+                    char *child_path = (char *)cli_malloc(size);
+                    if (child_path == NULL) {
+                        logg("*ClamInotif: could not allocate space for child path ... aborting\n");
+                        return NULL;
+                    }
+
+                    if (path[len - 1] == '/') {
+                        snprintf(child_path, --size, "%s%s", path, child);
+                    } else {
+                        snprintf(child_path, size, "%s/%s", path, child);
+                    }
+
+                    if (event->mask & IN_DELETE) {
+                        onas_ddd_handle_in_delete(ctx, path, child_path, event, wd);
+
+                    } else if (event->mask & IN_MOVED_FROM) {
+                        onas_ddd_handle_in_moved_from(ctx, path, child_path, event, wd);
+
+                    } else if (event->mask & IN_CREATE) {
+                        onas_ddd_handle_in_create(ctx, path, child_path, event, wd, in_mask);
+
+                    } else if (event->mask & IN_CLOSE_WRITE) {
+                        onas_ddd_handle_in_close_write(ctx, child_path);
+
+                    } else if (event->mask & IN_MOVED_TO) {
+                        onas_ddd_handle_in_moved_to(ctx, path, child_path, event, wd, in_mask);
+                    }
+
+                    free(child_path);
+                    child_path = NULL;
                 }
-
-                if (event->mask & IN_DELETE) {
-                    onas_ddd_handle_in_delete(ctx, path, child_path, event, wd);
-
-                } else if (event->mask & IN_MOVED_FROM) {
-                    onas_ddd_handle_in_moved_from(ctx, path, child_path, event, wd);
-
-                } else if (event->mask & IN_CREATE) {
-                    onas_ddd_handle_in_create(ctx, path, child_path, event, wd, in_mask);
-
-                } else if (event->mask & IN_CLOSE_WRITE) {
-                    onas_ddd_handle_in_close_write(ctx, child_path);
-
-                } else if (event->mask & IN_MOVED_TO) {
-                    onas_ddd_handle_in_moved_to(ctx, path, child_path, event, wd, in_mask);
-                }
-
-                free(child_path);
-                child_path = NULL;
             }
         }
     }
