@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm <tkojm@clamav.net>
@@ -98,14 +98,17 @@ static void sanitname(char *name)
 int cli_scancpio_old(cli_ctx *ctx)
 {
     struct cpio_hdr_old hdr_old;
+    char *fmap_name = NULL;
     char name[513];
     unsigned int file = 0, trailer = 0;
     uint32_t filesize, namesize, hdr_namesize;
     int ret         = CL_CLEAN, conv;
-    off_t pos       = 0;
+    size_t pos      = 0;
     int virus_found = 0;
 
-    while (fmap_readn(*ctx->fmap, &hdr_old, pos, sizeof(hdr_old)) == sizeof(hdr_old)) {
+    memset(name, 0, sizeof(name));
+
+    while (fmap_readn(ctx->fmap, &hdr_old, pos, sizeof(hdr_old)) == sizeof(hdr_old)) {
         pos += sizeof(hdr_old);
         if (!hdr_old.magic && trailer) {
             ret = CL_SUCCESS;
@@ -127,7 +130,7 @@ int cli_scancpio_old(cli_ctx *ctx)
         if (hdr_old.namesize) {
             hdr_namesize = EC16(hdr_old.namesize, conv);
             namesize     = MIN(sizeof(name), hdr_namesize);
-            if (fmap_readn(*ctx->fmap, &name, pos, namesize) != namesize) {
+            if (fmap_readn(ctx->fmap, &name, pos, namesize) != namesize) {
                 cli_dbgmsg("cli_scancpio_old: Can't read file name\n");
                 return CL_EFORMAT;
             }
@@ -144,6 +147,8 @@ int cli_scancpio_old(cli_ctx *ctx)
                 pos += hdr_namesize - namesize;
             } else if (hdr_namesize % 2)
                 pos++;
+
+            fmap_name = name;
         }
         filesize = (uint32_t)((uint32_t)EC16(hdr_old.filesize[0], conv) << 16 | EC16(hdr_old.filesize[1], conv));
         cli_dbgmsg("CPIO: Filesize: %u\n", filesize);
@@ -163,7 +168,7 @@ int cli_scancpio_old(cli_ctx *ctx)
             if (ret == CL_EMAXFILES) {
                 goto leave;
             } else if (ret == CL_SUCCESS) {
-                ret = cli_map_scan(*ctx->fmap, pos, filesize, ctx, CL_TYPE_ANY);
+                ret = cli_magic_scan_nested_fmap_type(ctx->fmap, pos, filesize, ctx, CL_TYPE_ANY, fmap_name);
                 if (ret == CL_VIRUS) {
                     if (!SCAN_ALLMATCHES)
                         return ret;
@@ -190,10 +195,12 @@ int cli_scancpio_odc(cli_ctx *ctx)
     unsigned int file = 0, trailer = 0;
     uint32_t filesize, namesize, hdr_namesize;
     int ret         = CL_CLEAN;
-    off_t pos       = 0;
+    size_t pos      = 0;
     int virus_found = 0;
 
-    while (fmap_readn(*ctx->fmap, &hdr_odc, pos, sizeof(hdr_odc)) == sizeof(hdr_odc)) {
+    memset(&hdr_odc, 0, sizeof(hdr_odc));
+
+    while (fmap_readn(ctx->fmap, &hdr_odc, pos, sizeof(hdr_odc)) == sizeof(hdr_odc)) {
         pos += sizeof(hdr_odc);
         if (!hdr_odc.magic[0] && trailer)
             goto leave;
@@ -215,7 +222,7 @@ int cli_scancpio_odc(cli_ctx *ctx)
         }
         if (hdr_namesize) {
             namesize = MIN(sizeof(name), hdr_namesize);
-            if (fmap_readn(*ctx->fmap, &name, pos, namesize) != namesize) {
+            if (fmap_readn(ctx->fmap, &name, pos, namesize) != namesize) {
                 cli_dbgmsg("cli_scancpio_odc: Can't read file name\n");
                 ret = CL_EFORMAT;
                 goto leave;
@@ -252,7 +259,7 @@ int cli_scancpio_odc(cli_ctx *ctx)
         if (ret == CL_EMAXFILES) {
             goto leave;
         } else if (ret == CL_SUCCESS) {
-            ret = cli_map_scan(*ctx->fmap, pos, filesize, ctx, CL_TYPE_ANY);
+            ret = cli_magic_scan_nested_fmap_type(ctx->fmap, pos, filesize, ctx, CL_TYPE_ANY, name);
             if (ret == CL_VIRUS) {
                 if (!SCAN_ALLMATCHES)
                     return ret;
@@ -276,12 +283,12 @@ int cli_scancpio_newc(cli_ctx *ctx, int crc)
     unsigned int file = 0, trailer = 0;
     uint32_t filesize, namesize, hdr_namesize, pad;
     int ret         = CL_CLEAN;
-    off_t pos       = 0;
+    size_t pos      = 0;
     int virus_found = 0;
 
     memset(name, 0, 513);
 
-    while (fmap_readn(*ctx->fmap, &hdr_newc, pos, sizeof(hdr_newc)) == sizeof(hdr_newc)) {
+    while (fmap_readn(ctx->fmap, &hdr_newc, pos, sizeof(hdr_newc)) == sizeof(hdr_newc)) {
         pos += sizeof(hdr_newc);
         if (!hdr_newc.magic[0] && trailer)
             goto leave;
@@ -303,7 +310,7 @@ int cli_scancpio_newc(cli_ctx *ctx, int crc)
         }
         if (hdr_namesize) {
             namesize = MIN(sizeof(name), hdr_namesize);
-            if (fmap_readn(*ctx->fmap, &name, pos, namesize) != namesize) {
+            if (fmap_readn(ctx->fmap, &name, pos, namesize) != namesize) {
                 cli_dbgmsg("cli_scancpio_newc: Can't read file name\n");
                 ret = CL_EFORMAT;
                 goto leave;
@@ -345,7 +352,7 @@ int cli_scancpio_newc(cli_ctx *ctx, int crc)
         if (ret == CL_EMAXFILES) {
             goto leave;
         } else if (ret == CL_SUCCESS) {
-            ret = cli_map_scan(*ctx->fmap, pos, filesize, ctx, CL_TYPE_ANY);
+            ret = cli_magic_scan_nested_fmap_type(ctx->fmap, pos, filesize, ctx, CL_TYPE_ANY, name);
             if (ret == CL_VIRUS) {
                 if (!SCAN_ALLMATCHES)
                     return ret;
