@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2013 Sourcefire, Inc.
  *
  *  Authors: David Raynor <draynor@sourcefire.com>
@@ -54,7 +54,8 @@ static cl_error_t hfsplus_scanfile(cli_ctx *, hfsPlusVolumeHeader *, hfsHeaderRe
                                    hfsPlusForkData *, const char *, char **, char *);
 static int hfsplus_validate_catalog(cli_ctx *, hfsPlusVolumeHeader *, hfsHeaderRecord *);
 static int hfsplus_fetch_node(cli_ctx *, hfsPlusVolumeHeader *, hfsHeaderRecord *,
-                              hfsHeaderRecord *, hfsPlusForkData *, uint32_t, uint8_t *);
+                              hfsHeaderRecord *, hfsPlusForkData *, uint32_t, uint8_t *,
+                              size_t);
 static cl_error_t hfsplus_walk_catalog(cli_ctx *, hfsPlusVolumeHeader *, hfsHeaderRecord *,
                                        hfsHeaderRecord *, hfsHeaderRecord *, const char *);
 
@@ -138,11 +139,11 @@ static int hfsplus_volumeheader(cli_ctx *ctx, hfsPlusVolumeHeader **header)
     }
 
     /* Start with volume header, 512 bytes at offset 1024 */
-    if ((*ctx->fmap)->len < 1536) {
+    if (ctx->fmap->len < 1536) {
         cli_dbgmsg("hfsplus_volumeheader: too short for HFS+\n");
         return CL_EFORMAT;
     }
-    mPtr = fmap_need_off_once(*ctx->fmap, 1024, 512);
+    mPtr = fmap_need_off_once(ctx->fmap, 1024, 512);
     if (!mPtr) {
         cli_errmsg("hfsplus_volumeheader: cannot read header from map\n");
         return CL_EMAP;
@@ -244,7 +245,7 @@ static int hfsplus_readheader(cli_ctx *ctx, hfsPlusVolumeHeader *volHeader, hfsN
             cli_errmsg("hfsplus_readheader: %s: invalid headerType %d\n", name, headerType);
             return CL_EARG;
     }
-    mPtr = fmap_need_off_once(*ctx->fmap, offset, volHeader->blockSize);
+    mPtr = fmap_need_off_once(ctx->fmap, offset, volHeader->blockSize);
     if (!mPtr) {
         cli_dbgmsg("hfsplus_readheader: %s: headerNode is out-of-range\n", name);
         return CL_EFORMAT;
@@ -400,7 +401,7 @@ static cl_error_t hfsplus_scanfile(cli_ctx *ctx, hfsPlusVolumeHeader *volHeader,
             size_t written;
             off_t offset = currBlock * volHeader->blockSize;
             /* move map to next block */
-            mPtr = fmap_need_off_once(*ctx->fmap, offset, volHeader->blockSize);
+            mPtr = fmap_need_off_once(ctx->fmap, offset, volHeader->blockSize);
             if (!mPtr) {
                 cli_errmsg("hfsplus_scanfile: map error\n");
                 ret = CL_EMAP;
@@ -521,7 +522,7 @@ static cl_error_t hfsplus_check_attribute(cli_ctx *ctx, hfsPlusVolumeHeader *vol
         }
 
         /* fetch node into buffer */
-        ret = hfsplus_fetch_node(ctx, volHeader, attrHeader, NULL, &(volHeader->attributesFile), thisNode, nodeBuf);
+        ret = hfsplus_fetch_node(ctx, volHeader, attrHeader, NULL, &(volHeader->attributesFile), thisNode, nodeBuf, nodeSize);
         if (ret != CL_CLEAN) {
             cli_dbgmsg("hfsplus_check_attribute: node fetch failed.\n");
             break;
@@ -629,7 +630,8 @@ static cl_error_t hfsplus_check_attribute(cli_ctx *ctx, hfsPlusVolumeHeader *vol
 
 /* Fetch a node's contents into the buffer */
 static int hfsplus_fetch_node(cli_ctx *ctx, hfsPlusVolumeHeader *volHeader, hfsHeaderRecord *catHeader,
-                              hfsHeaderRecord *extHeader, hfsPlusForkData *catFork, uint32_t node, uint8_t *buff)
+                              hfsHeaderRecord *extHeader, hfsPlusForkData *catFork, uint32_t node, uint8_t *buff,
+                              size_t buffSize)
 {
     int foundBlock = 0;
     uint64_t catalogOffset;
@@ -714,7 +716,12 @@ static int hfsplus_fetch_node(cli_ctx *ctx, hfsPlusVolumeHeader *volHeader, hfsH
             readSize = endSize;
         }
 
-        if (fmap_readn(*ctx->fmap, buff + buffOffset, fileOffset, readSize) != readSize) {
+        if ((buffOffset + readSize) > buffSize) {
+            cli_dbgmsg("hfsplus_fetch_node: Not enough space for read\n");
+            return CL_EFORMAT;
+        }
+
+        if (fmap_readn(ctx->fmap, buff + buffOffset, fileOffset, readSize) != readSize) {
             cli_dbgmsg("hfsplus_fetch_node: not all bytes read\n");
             return CL_EFORMAT;
         }
@@ -911,7 +918,7 @@ static cl_error_t hfsplus_walk_catalog(cli_ctx *ctx, hfsPlusVolumeHeader *volHea
         }
 
         /* fetch node into buffer */
-        ret = hfsplus_fetch_node(ctx, volHeader, catHeader, extHeader, &(volHeader->catalogFile), thisNode, nodeBuf);
+        ret = hfsplus_fetch_node(ctx, volHeader, catHeader, extHeader, &(volHeader->catalogFile), thisNode, nodeBuf, nodeSize);
         if (ret != CL_SUCCESS) {
             cli_dbgmsg("hfsplus_walk_catalog: node fetch failed.\n");
             break;
