@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Author: aCaB
@@ -37,8 +37,8 @@
 #include <sys/socket.h>
 #endif
 
-#include "shared/output.h"
-#include "shared/clamdcom.h"
+#include "output.h"
+#include "clamdcom.h"
 
 /* Sends bytes over a socket
  * Returns 0 on success */
